Attribute VB_Name = "modPlugins070"
' the plugin system for EEShell2
' got the idea from an article somewhere on the WWW about (surprise) how to make
' a plugin system for VB apps

' copyright 2004-2009 Erik Siers
' source code provided under terms of version 2 of the GNU GPL

' 20080929 EES:
'   Should consider making this a Class.

Option Explicit

' enums, vars, and type private to avoid rest of app somehow going haywire and
' sending signals to plugins -- all comms w/plugins must happen through this
' module

' signal constants
Public Const pSTART As Long = 1&    ' start the plugin (already loaded)
Public Const pREFRESH As Long = 2&  ' sent every time background refreshes
Public Const pSTOP As Long = 3&     ' stop the plugin (not necessarily unload)

' response constants
Private Const rOK As Long = 0&          ' success; no error
Private Const rUNHANDLED As Long = 1&   ' unhandled signal; not an error
Private Const rERROR As Long = 2&       ' error condition
Private Const rREQUEST As Long = 4&     ' plugin needs more info to work

Private Type Plugin
    pObj As Object
    pName As String
    pFile As String
End Type

' pCount = 0 -> no plugins -- first plugin is #1, plugin #0 is null (but included)
Private pCount As Long
Private Plugins() As Plugin

Public Function listPlugins() As String()
    ' returns a string array of the names of all valid plugins

    Dim n As Long
    ReDim outP(pCount) As String

    For n = 1 To pCount
        outP(n) = Plugins(n).pName
    Next
    listPlugins = outP
End Function

Public Sub startAllPlugins()
    ' sends pSTART signal to all plugins

    Dim n As Long

    For n = 1 To pCount
        SignalPlugin n, pSTART
    Next
End Sub

Public Sub refreshAllPlugins()
    ' sends pREFRESH signal to all plugins

    Dim n As Long

    For n = 1 To pCount
        SignalPlugin n, pREFRESH
    Next
End Sub

Public Sub stopAllPlugins()
    ' sends pSTOP signal to all plugins

    Dim n As Long

    For n = 1 To pCount
        SignalPlugin n, pSTOP
    Next
End Sub

Public Sub findPlugins()
    ' looks in plugins directory and finds all .dll files
    ' those that are registered get mapped into Plugins() array
    ' those that aren't registered *should* be ignored, but for some reason they
    ' somethimes *aren't* ignored

    ' can't do this more than once per instance
    Dim z As Long
    If pCount > 0 Then
        z = appLog("Error: tried to load plugins with pCount > 0", "findPlugins")
        Exit Sub
    End If

    Dim x As String, y As String, curD As String, curDr As String * 1

    curD = CurDir
    curDr = Left(curD, 1)
    ChDrive Left(App.Path, 1)
    ChDir App.Path & "\plugins"
    x = Dir("*.dll")
    Do While x <> ""
        y = Left(x, InStr(x, ".") - 1)
        If isValidPlugin(y) Then
            pCount = pCount + 1
            ReDim Preserve Plugins(pCount)
            Plugins(pCount).pFile = App.Path & "\Plugins\" & x
            Plugins(pCount).pName = y
            z = appLog("Found plugin: " & y & " in " & Plugins(pCount).pFile, "findPlugins")
        End If
        x = Dir
    Loop
    loadPlugins
    ChDrive curDr
    ChDir curD
End Sub

Public Sub loadPlugins()
    ' loads all plugins into memory by creating an object from each DLL
    ' if we get here, plugin should already have been checked for being registered

    Dim f As String, n As Long
    For n = 1 To pCount
        f = Plugins(n).pName & ".Plugin"
        Set Plugins(n).pObj = CreateObject(f)
    Next
End Sub

Sub unloadPlugins()
    ' unloads all plugins by erasing the object created from each DLL

    Dim n As Long
    For n = pCount To 1 Step -1
        Set Plugins(n).pObj = Nothing
    Next
    pCount = 0
End Sub

Public Sub SignalPlugin(Plug As Long, Sig As Long)
    ' sends signal to a single plugin; finds it by index #
    ' you can get the index # by calling listPlugins()

    On Error GoTo crapSP
    If (Sig < 1) Or (Sig > 3) Then Err.Raise 1, , "Invalid signal"
    Dim pr As Long
    pr = Plugins(Plug).pObj.Signal(Sig)
    Select Case pr
        Case rOK
            ' started okay; no further action needed
        Case rUNHANDLED
            ' unhandled signal; shouldn't get this with pSTART
            Dim o As String
            Select Case Sig
                Case pSTART
                    o = "pSTART"
                Case pSTOP
                    o = "pSTOP"
                Case pREFRESH
                    o = "pREFRESH"
                Case Else
                    ' for future signals
                    ' not an error if not supported by plugin
                    ' will need to handle this
                    o = ""
            End Select
            If Len(o) Then Err.Raise 2, , "Plugin doesn't know how to handle " & o
        Case rERROR
            ' plugin hit a snag while processing the signal
            ' need to figure what to do to handle this
            Err.Raise 4, , "Plugin returned rERROR"
    End Select
    Exit Sub

crapSP:
    Dim d As String, n As Long
    n = Err.Number: d = Err.Description
    Err.Clear
    n = appLog("Error #" & CStr(n) & " sending signal " & Sig & " to plugin " & _
            Plugins(Plug).pName & ": " & d, "SignalPlugin")
End Sub

Private Function isValidPlugin(DLL As String) As Boolean
    ' determines if plugin has been registered properly

    On Error Resume Next
    Dim x As Object, n As String

    n = DLL & ".Plugin"
    Set x = CreateObject(n)
    If Nothing Is x Then    ' might try -> isValidPlugin = (Nothing is x)
        isValidPlugin = False
    Else
        isValidPlugin = True
    End If
    Set x = Nothing
End Function
