' Image selection routines for EEShell2.dll.
' Requires:
'   Win32API.bi (for a constant)

' Copyright 2004-2010 Erik Siers
' EEShell2 is a complete, functional replacement shell for Windows.
' For details, usage & license, see the file "EEShell2.txt"


GLOBAL fileLS() AS STRING, currentImage AS LONG, totalImages AS LONG


MACRO fixSlash(PATH) = IF RIGHT$(PATH, 1) <> "\" THEN PATH = PATH & "\"
'SUB fixSlash(BYREF path AS STRING)
    ' adds a trailing "\" if not already there
    ' args:
    '   path    do I really need to explain this?
    ' called by:
    '   checkList
    '   makeImgList
    '   randPic
    '   recurseDirs

'    IF RIGHT$(path, 1) <> "\" THEN path = path & "\"
'END SUB


SUB setInitialValues()
    ' sets shared variables to initial values
    ' called by:
    '   LIBMAIN     (EEShell2.bas)

    currentImage = -1
    totalImages = -1
END SUB


SUB closeImages()
    ' erases fileLS prior to unloading DLL
    ' called by:
    '   LIBMAIN     (EEShell2.bas)

    ERASE fileLS()
END SUB


SUB recurseDirs (BYVAL folder AS STRING, chop AS LONG, BYREF UBfileLS AS LONG)
    ' follows a single branch all the way (stack space, don't fail me now)
    ' args:
    '   folder      folder we're working in
    '   chop        how much of the lead to chop off before jamming into fileLS
    '   UBfileLS    ubound(fileLS)
    ' calls:
    '   fixSlash
    '   recurseDirs ***RECURSIVE***
    '   (might need to make some API calls before I'm done...)
    ' called by:
    '   makeImgList

    ON ERROR GOTO Whoops

    LOCAL tmpStr1 AS STRING, tmpLng1 AS LONG, fldrs AS LONG, EXTN AS STRING, lFiles AS LONG
    LOCAL subdirs() AS STRING, localFiles() AS STRING
    LOCAL counter AS LONG

    fldrs = -1: lFiles = -1
    fixSlash(folder)

    tmpStr1 = DIR$(folder, %FILE_ATTRIBUTE_DIRECTORY)
    DO UNTIL "" = tmpStr1
        IF ("." = tmpStr1) OR (".." = tmpStr1) THEN GOTO Looper
        tmpLng1 = GETATTR(folder & tmpStr1)
        IF %FILE_ATTRIBUTE_DIRECTORY = (tmpLng1 AND %FILE_ATTRIBUTE_DIRECTORY) THEN
            INCR fldrs
            REDIM PRESERVE subdirs(fldrs)
            subdirs(fldrs) = folder & tmpStr1
        ELSE
            'build localFiles, to be appended to fileLS later
            tmpLng1 = INSTR(-1, tmpStr1, ".")
            IF 0 = tmpLng1 THEN GOTO Looper
            EXTN = LCASE$(MID$(tmpStr1, tmpLng1 + 1))
            SELECT CASE EXTN
#IF 1 = %usePNG
' I wish
                CASE "bmp", "emf", "gif", "jpe", "jpeg", "jpg", "pcx", "png", "wmf"
#ELSE
                CASE "bmp", "emf", "gif", "jpe", "jpeg", "jpg", "wmf"
#ENDIF
                    INCR lFiles
                    REDIM PRESERVE localFiles(lFiles)
                    localFiles(lFiles) = tmpStr1
            END SELECT
        END IF
Looper:
        tmpStr1 = DIR$
        INCR counter
        IF 0 = (counter MOD 10) THEN
            counter = 0
            SLEEP 0
        END IF
    LOOP

    IF lFiles > -1 THEN
        ARRAY SORT localFiles(), COLLATE UCASE
        REDIM PRESERVE fileLS(UBfileLS + lFiles + 1)
        tmpStr1 = MID$(folder, chop)
        FOR tmpLng1 = 0 TO lFiles
            fileLS (UBfileLS + tmpLng1 + 1) = tmpStr1 & localFiles(tmpLng1)
        NEXT
        UBfileLS = UBfileLS + lFiles + 1
    END IF

    IF fldrs > -1 THEN
        ARRAY SORT subdirs(), COLLATE UCASE
        FOR tmpLng1 = 0 TO fldrs
            recurseDirs subdirs(tmpLng1), chop, UBfileLS
        NEXT
    END IF

Whoops:
    ERRCLEAR
END SUB


FUNCTION makeImgList ALIAS "makeImgList" (PATH AS STRING) AS LONG
    ' starts the ball rolling on building the list
    ' args:
    '   path    base image path
    ' calls:
    '   recurseDirs
    ' called by:
    '   checkList

    ON ERROR GOTO noimages
    LOCAL c AS LONG, lh AS LONG
    LOCAL cdir AS STRING, lc AS LONG, c1 AS STRING
    LOCAL fi AS LONG

    ERASE fileLS()

    cdir = PATH
    fixSlash(cdir)
    lc = LEN(cdir) + 1

    c = -1
    lh = LEN(cdir) + 1
    recurseDirs cdir, lh, c
    IF c < 0 THEN EXIT FUNCTION

    c1 = CURDIR$
    CHDIR cdir
    fi = FREEFILE
    totalImages = c + 1
noimages:
    CHDIR c1
    FUNCTION = ERRCLEAR
END FUNCTION


FUNCTION checkList (PATH AS STRING) AS LONG
    ' makes sure the list exists; if not, rebuilds it
    ' args:
    '   path    base image path
    ' calls:
    '   makeImgList
    ' called by:
    '   nextPic
    '   prevPic
    '   randPic

    STATIC p AS STRING
    LOCAL tmpLng1 AS LONG
    fixSlash(PATH)
    PATH = UCASE$(PATH)
    IF (p <> PATH) OR (totalImages < 0) THEN
        p = PATH
        tmpLng1 = makeImgList (p)
        FUNCTION = tmpLng1
    END IF
END FUNCTION


FUNCTION randPic ALIAS "randPic" (PATH AS STRING) EXPORT AS STRING
    ' selects a random entry in the image list
    ' args:
    '   path    base image path
    ' calls:
    '   checkList
    ' called by:
    '   EEShell2.exe->frmBG.cmdImgRand_Click
    '   EEShell2.exe->frmBG.resizeForm
    '   EEShell2.exe->frmBG.tmrRandImg_Timer
    '   EEShell2.exe->frmOpts.cmdBrowse1_Click
    '   EEShell2.exe->frmOpts.Form_Load

    LOCAL l AS LONG
    l = checkList (PATH)
    IF l > 0 THEN EXIT FUNCTION
    RANDOMIZE TIMER
    fixSlash(PATH)
    currentImage = INT(RND * totalImages)
    FUNCTION = PATH & fileLS(currentImage)
END FUNCTION


FUNCTION nextPic ALIAS "nextPic" (PATH AS STRING) EXPORT AS STRING
    ' selects the next entry in the image list, wrapping to 0 if at ubound
    ' args:
    '   path    base image path
    ' calls:
    '   checkList
    ' called by:
    '   EEShell2.exe->frmBG.cmdImgNext_Click

    LOCAL l AS LONG
    l = checkList (PATH)
    IF l > 0 THEN EXIT FUNCTION
    INCR currentImage
    IF currentImage >= totalImages THEN currentImage = 0
    FUNCTION = PATH & fileLS(currentImage)
END FUNCTION


FUNCTION prevPic ALIAS "prevPic" (PATH AS STRING) EXPORT AS STRING
    ' selects the previous entry in the image list, wrapping to ubound if at 0
    ' args:
    '   path    base image path
    ' calls:
    '   checkList
    ' called by:
    '   EEShell2.exe->frmBG.cmdImgPrev_Click

    LOCAL l AS LONG
    l = checkList (PATH)
    IF l > 0 THEN EXIT FUNCTION
    DECR currentImage
    IF currentImage < 0 THEN currentImage = totalImages - 1
    FUNCTION = PATH & fileLS(currentImage)
END FUNCTION


' this isn't used anywhere, so commenting out
' if you need it, uncomment and recompile
'FUNCTION getCurrentPic ALIAS "getCurrentPic" () EXPORT AS STRING
    ' returns the current entry in the image list

'    FUNCTION = fileLS(currentImage)
'END FUNCTION
