' The run module for EEShell2.dll.
' requires:
'   EEShell2.bas
'   Win32API.bi
' required by:
'   Startup.bi

' Copyright 2004-2010 Erik Siers
' EEShell2 is a complete, functional replacement shell for Windows.
' For details, usage & license, see the file "EEShell2.txt"


FUNCTION resEnv ALIAS "resEnv" (pPath AS STRING) EXPORT AS STRING
    ' resolves environment variables
    ' args:
    '   pPath   the string containing env.vars to resolve
    '           begins with env.var, need to find first %, chop, resolve
    '           if win32, limited to 64k for now
    ' calls:
    '   ExpandEnvironmentStrings    (Win32API.bi->kernel32)
    '   resEnv ***RECURSIVE***      (only if not win32)
    ' called by:
    '   Run

    DIM tmpLng1 AS LONG

$IF %DEF (%PB_WIN32)    ' ExpandEnvironmentStrings requires NT3.1+ or 95+; not available in Win32s
    DIM tmpStr1 AS ASCIIZ * 65535, tmpStr2 AS ASCIIZ * %MAX_PATH
    tmpStr1 = pPath
    tmpLng1 = ExpandEnvironmentStrings (tmpStr1, tmpStr2, %MAX_PATH)
    '*****to do: need to handle error if either tmpStr isn't big enough...
$ELSE
    DIM tmpLng2 AS LONG, tmpStr1 AS STRING, tmpStr2 AS STRING
    tmpLng1 = INSTR(pPath, "%")
    IF tmpLng1 THEN
        tmpStr1 = LEFT$(pPath, tmpLng1 - 1)
        tmpLng2 = INSTR(tmpLng1 + 1, pPath, "%")
        IF tmpLng2 THEN
            tmpStr2 = ENVIRON$(tmpStr1) & MID$(pPath, tmpLng1 + 1, tmpLng2 - tmpLng1 - 1) & resEnv(MID$(pPath, tmpLng2 + 1))
        ELSE
            tmpStr2 = ENVIRON$(tmpStr1) & MID$(pPath, tmpLng1 + 1)
        END IF
    END IF
$ENDIF

    resEnv = tmpStr2
END FUNCTION


SUB Run ALIAS "Run" (runItem AS ASCIIZ, cmdLine AS ASCIIZ, defPath AS ASCIIZ) EXPORT
    ' launches runItem
    ' args:
    '   runItem     the item to launch
    '   cmdLine     args to runItem
    '   defPath     default path for runItem
    ' calls:
    '   appLog      (EEShell2.bas)
    '   resEnv
    ' called by:
    '   runStartup  (Startup.bi)
    '   EEShell2.exe->frmBG.cmdFileMan_Click
    '   EEShell2.exe->frmBG.cmdRun_Click

    'to do: allow searching of %path%
    'right now, item must be the full path to whatever is being started
    'environment variables dealt with
    ON ERROR GOTO ohCrapRun
    DIM sE AS LONG

    'resolve environment variables
    sE = INSTR(runItem, "%")
    IF sE THEN runItem = LEFT$(runItem, sE - 1) & resEnv(MID$(runItem, sE + 1))

    'check and set default path in case missing path breaks app
    '(known issue with Black Isle games and some other apps)
    IF LEN(defPath) < 1 THEN
        sE = INSTR(-1, runItem, "\")
        IF sE THEN defPath = LEFT$(runItem, sE - 1)
    END IF

    sE = ShellExecute(0, "", runItem, cmdLine, defPath, 5)
#IF %DEF (%dBug)
    IF sE <= 32 THEN appLog "ShellExecute: Error #" & STR$(sE)
#ENDIF
    IF 2 = sE THEN MSGBOX "File not found."
    EXIT SUB

ohCrapRun:
    DIM x AS ASCIIZ * 256
    sE = ERRCLEAR
    IF sE THEN
        x = "Error " & STR$(sE) & " in Run: " & ERROR$(sE)
    ELSE
        sE = 0
        FormatMessage %FORMAT_MESSAGE_FROM_SYSTEM, sE, ERRAPI, 0, x, 256, sE
    END IF
    DIM x2 AS STRING
    x2 = x
#IF %DEF (%dBug)
    appLog x2
#ENDIF
END SUB
