' The shell module for EEShell2.dll.
' requires:
'   EEShell2.bas
'   Win32API.bi
' required by:
'   Startup.bi

' Copyright 2004-2010 Erik Siers
' EEShell2 is a complete, functional replacement shell for Windows.
' For details, usage & license, see the file "EEShell2.txt"


$SYSBootKey = "SOFTWARE\Microsoft\Windows NT\CurrentVersion\IniFileMapping\system.ini\boot"
$BootKeyShellValue = "Shell"


FUNCTION GetKeyValue ALIAS "GetKeyValue" (KeyRoot AS LONG, KeyName AS ASCIIZ, SubKeyRef AS ASCIIZ, BYREF KeyVal AS ASCIIZ) _
        EXPORT AS LONG
    ' retreives a registry value (wrapper for RegQueryValueEx)
    ' args:
    '   KeyRoot     the reg hive
    '   KeyName     the reg path
    '   SubKeyRef   the key name
    '   KeyVal      this is where the returned information gets stuffed
    ' calls:
    '   appLog          (EEShell.bas)
    '   FormatMessage   (Win32API.bi->kernel32)
    '   RegOpenKeyEx    (Win32API.bi->advapi32)
    '   RegQueryValueEx (Win32API.bi->advapi32)
    '   RegCloseKey     (Win32API.bi->advapi32)
    ' called by:
    '   getSystemShellNT
    '   setSystemShellNT

    DIM rc AS LONG
    DIM hKey AS LONG
    DIM hDepth AS LONG
    DIM KeyValType AS LONG
    DIM tmpVal AS ASCIIZ * 256
    DIM KeyValSize AS LONG

    ' Open RegKey Under KeyRoot {HKEY_LOCAL_MACHINE...}
    rc = RegOpenKeyEx(KeyRoot, KeyName, 0, %KEY_ALL_ACCESS, hKey)

    IF (rc <> %ERROR_SUCCESS) THEN GOTO GetKeyError

    KeyValSize = 255

    ' Retrieve Registry Key Value...
    rc = RegQueryValueEx(hKey, SubKeyRef, 0, KeyValType, tmpVal, KeyValSize)

    IF (rc <> %ERROR_SUCCESS) THEN GOTO GetKeyError

    ' Determine Key Value Type For Conversion...
    SELECT CASE KeyValType
        CASE %REG_SZ
            KeyVal = tmpVal
        CASE ELSE
            ERROR 151
    END SELECT

    rc = RegCloseKey(hKey)
    FUNCTION = -1
    EXIT FUNCTION

GetKeyError:
    hDepth = ERRCLEAR
    DIM a1 AS STRING, a2 AS STRING
    a1 = STR$(KeyRoot)
    a2 = KeyName
    IF hDepth THEN
        tmpVal = ERROR$(hDepth)
        appLog "Error " & STR$(hDepth) & " while reading " & a1 & "\" & a2 & ": " & tmpVal & " (rc=" & STR$(rc) & ")"
    ELSE
        DIM a3 AS LONG
        a3 = 0
        FormatMessage %FORMAT_MESSAGE_FROM_SYSTEM, a3, ERRAPI, 0, tmpVal, 255, a3
        appLog "Error while reading " & a1 & "\" & a2 & ": " & TRIM$(tmpVal) & " (rc=" & STR$(rc) & ")"
    END IF
    rc = RegCloseKey(hKey)
    KeyVal = "ERROR"
    FUNCTION = 0
END FUNCTION


FUNCTION SetKeyValue ALIAS "SetKeyValue" (KeyRoot AS LONG, KeyName AS ASCIIZ, SubKeyRef AS ASCIIZ, BYREF KeyVal AS ASCIIZ) _
        EXPORT AS LONG
    ' sets a registry value (wrapper for RegSetValueEx)
    ' args:
    '   KeyRoot     the reg hive
    '   KeyName     the reg path
    '   SubKeyRef   the key name
    '   KeyVal      the new value
    ' calls:
    '   appLog          (EEShell.bas)
    '   FormatMessage   (Win32API.bi->kernel32)
    '   RegOpenKeyEx    (Win32API.bi->advapi32)
    '   RegSetValueEx   (Win32API.bi->advapi32)
    '   RegCloseKey     (Win32API.bi->advapi32)
    ' called by:
    '   setSystemShellNT

    DIM rc AS LONG
    DIM hKey AS LONG
    DIM hDepth AS LONG
    DIM KeyValType AS LONG
    DIM tmpVal AS ASCIIZ * 256
    DIM KeyValSize AS LONG

    ' Open RegKey Under KeyRoot {HKEY_LOCAL_MACHINE...}
    rc = RegOpenKeyEx(KeyRoot, KeyName, 0, %KEY_ALL_ACCESS, hKey)

    IF (rc <> %ERROR_SUCCESS) THEN GOTO GetKeyError

    KeyValSize = 255

    ' Set Registry Key Value...
    KeyValType = %REG_SZ
    rc = RegSetValueEx(hKey, SubKeyRef, 0, KeyValType, KeyVal, KeyValSize)

    IF (rc <> %ERROR_SUCCESS) THEN GOTO GetKeyError

    rc = RegCloseKey(hKey)
    FUNCTION = -1
    EXIT FUNCTION

GetKeyError:
    hDepth = ERRCLEAR
    DIM a1 AS STRING, a2 AS STRING
    a1 = STR$(KeyRoot)
    a2 = KeyName
    IF hDepth THEN
        tmpVal = ERROR$(hDepth)
        appLog "Error " & STR$(hDepth) & " while reading " & a1 & "\" & a2 & ": " & tmpVal & " (rc=" & STR$(rc) & ")"
    ELSE
        DIM a3 AS LONG
        a3 = 0
        FormatMessage %FORMAT_MESSAGE_FROM_SYSTEM, a3, ERRAPI, 0, tmpVal, 255, a3
        appLog "Error while reading " & a1 & "\" & a2 & ": " & TRIM$(tmpVal) & " (rc=" & STR$(rc) & ")"
    END IF
    rc = RegCloseKey(hKey)
    KeyVal = "ERROR"
    FUNCTION = 0
END FUNCTION


FUNCTION getINIvalue ALIAS "getINIvalue" (file AS ASCIIZ, section AS ASCIIZ, key AS ASCIIZ, value AS STRING) EXPORT AS LONG
    ' reads a value from a .ini file (wrapper for GetPrivateProfileString)
    ' args:
    '   file    the .ini file to look in
    '   section the [section] to look under
    '   key     the key= to look at
    '   value   this is where the returned information gets stuffed
    ' calls:
    '   GetPrivateProfileString (Win32API.bi->kernel32)
    ' called by:
    '   getSystemShell9x
    '   EEShell2.exe->modINI.readINI

    LOCAL res AS ASCIIZ * 256, resval AS DWORD

    resval = GetPrivateProfileString(section, key, "", res, 256???, file)
    value = LEFT$(res, resval)
    FUNCTION = resval
END FUNCTION


FUNCTION writeINIvalue ALIAS "writeINIvalue" (file AS ASCIIZ, section AS ASCIIZ, key AS ASCIIZ, value AS ASCIIZ) _
        EXPORT AS LONG
    ' writes a value to a .ini file (wrapper for WritePrivateProfileString)
    ' args:
    '   file    the .ini file to look in
    '   section the [section] to look under
    '   key     the key= to look at
    '   value   what to assign to key
    ' calls:
    '   WritePrivateProfileString (Win32API.bi->kernel32)
    ' called by:
    '   setSystemShell9x
    '   EEShell2.exe->modINI.writeINI

    LOCAL res AS ASCIIZ * 256, resval AS DWORD

    resval = WritePrivateProfileString(section, key, value, file)
    FUNCTION = resval   ' 0 = failure, anything else is success
END FUNCTION


'FUNCTION getSystemShell9x() AS STRING
'    ' reads the current shell from %windir%\system.ini
'    ' calls:
'    '   getINIvalue
'    ' called by:
'    '   getSystemShell
'
'    LOCAL res AS STRING, tmpLng AS LONG
'    LOCAL tmp1 AS ASCIIZ * 5, tmp2 AS ASCIIZ * 6, tmp3 AS ASCIIZ * 11
'
'    tmp1 = "boot"
'    tmp2 = $BootKeyShellValue
'    tmp3 = "System.ini"
'
'    tmpLng = getINIvalue(tmp3, tmp1, tmp2, res)
'    FUNCTION = res
'END FUNCTION


FUNCTION setSystemShell9x(what AS ASCIIZ) AS LONG
    ' sets the shell in %windir%\system.ini
    ' args:
    '   what    what to set as the new shell
    '           Win32: should be "Explorer.exe" or the full path to EEShell2
    '           Win16: should be "Progman.exe" or the full path to EEShell2/16
    ' calls:
    '   writeINIvalue
    ' called by:
    '   setSystemShell

    LOCAL retval AS LONG
    LOCAL tmp1 AS ASCIIZ * 5, tmp2 AS ASCIIZ * 6, tmp3 AS ASCIIZ * 11

    tmp1 = "boot"
    tmp2 = $BootKeyShellValue
    tmp3 = "System.ini"

    retval = writeINIvalue(tmp3, tmp1, tmp2, what)
    FUNCTION = retval
END FUNCTION


'FUNCTION getSystemShellNT() AS STRING
'    ' reads the current shell from the registry
'    ' calls:
'    '   GetKeyValue
'    ' called by:
'    '   getSystemShell
'
'    DIM c AS LONG
'    DIM currentShell AS ASCIIZ * %MAX_PATH, usrShell AS ASCIIZ * %MAX_PATH
'
'    c = GetKeyValue(%HKEY_LOCAL_MACHINE, $SYSBootKey, $BootKeyShellValue, currentShell)
'    IF c THEN
'        SELECT CASE UCASE$(LEFT$(currentShell, 3))
'            CASE "USR"
'                c = GetKeyValue(%HKEY_CURRENT_USER, currentShell, $BootKeyShellValue, usrShell)
'                IF c THEN currentShell = usrShell
'            CASE "SYS"
'                c = GetKeyValue(%HKEY_LOCAL_MACHINE, currentShell, $BootKeyShellValue, usrShell)
'                IF c THEN currentShell = usrShell
'            CASE ELSE
'                currentShell = "ERROR"
'        END SELECT
'        FUNCTION = currentShell
'    END IF
'END FUNCTION


FUNCTION setSystemShellNT(what AS ASCIIZ) AS LONG
    ' changes the shell in the registry
    ' args:
    '   what    the new shell
    '           should be either "Explorer.exe" or the full path to EEShell2
    ' calls:
    '   GetKeyValue
    '   SetKeyValue
    ' called by:
    '   setSystemShell

    DIM c AS LONG
    DIM currentShell AS ASCIIZ * %MAX_PATH, usrShell AS ASCIIZ * %MAX_PATH

    c = GetKeyValue(%HKEY_LOCAL_MACHINE, $SYSBootKey, $BootKeyShellValue, currentShell)
    IF c THEN
        SELECT CASE UCASE$(LEFT$(currentShell, 3))
            CASE "USR"
                c = SetKeyValue(%HKEY_CURRENT_USER, "Software\Microsoft\Windows NT\CurrentVersion\Winlogon", $BootKeyShellValue, what)
                appLog "Set EEShell2 as shell."
                FUNCTION = c
            CASE "SYS"
                IF ("EXPLORER.EXE" = UCASE$(what)) OR ("EXPLORER" = UCASE$(what)) THEN
                    ' I expect this to fail if the current user isn't admin
                    c = SetKeyValue(%HKEY_LOCAL_MACHINE, "Software\Microsoft\Windows NT\CurrentVersion\Winlogon", $BootKeyShellValue, what)
                    FUNCTION = c
                ELSE
                    MSGBOX "EEShell2 will not set itself as the shell for all users under NT systems." & $CRLF & _
                           "Import the registry file ""1-admin-install.reg"" (in the EEShell2 directory)" & $CRLF & _
                           "and then try again.", %MB_ICONWARNING OR %MB_TASKMODAL
                    appLog "Failed to set EEShell2 as shell."
                    FUNCTION = 0    'failure
                END IF
            CASE ELSE
                MSGBOX "An unexpected registry value was encountered while trying to set EEShell2 as the shell." & $CRLF & _
                       "EEShell2 will not be able to automatically set itself as your shell on this system." & $CRLF & _
                       "Read the file ""Shell.txt"" in the EEShell2 directory for manual instructions.", %MB_TASKMODAL OR _
                       %MB_ICONERROR
                FUNCTION = 0    'failure
        END SELECT
    END IF
END FUNCTION


'FUNCTION getSystemShell ALIAS "getSystemShell" () EXPORT AS STRING
'    ' wrapper for getSystemShell9x and getSystemShellNT
'    ' calls:
'    '   getSystemShell9x
'    '   getSystemShellNT
'    '   IsWinNT
'    ' called by:
'    '   nothing, at the moment, so commented out
'
'    IF ISTRUE(IsWinNT) THEN
'        FUNCTION = getSystemShellNT
'    ELSE
'        FUNCTION = getSystemShell9x
'    END IF
'END FUNCTION


FUNCTION setSystemShell ALIAS "setSystemShell" (what AS ASCIIZ) EXPORT AS LONG
    ' changes the shell is a system-specific manner
    ' (wrapper for setSystemShell9x and setSystemShellNT)
    ' args:
    '   what    the new shell
    ' calls:
    '   IsWinNT             (EEShell.bas)
    '   setSystemShell9x
    '   setSystemShellNT
    ' called by:
    '   EEShell2.exe->frmOpts.cmdOK_Click

    IF ISTRUE(IsWinNT) THEN
        FUNCTION = setSystemShellNT (what)
    ELSE
        FUNCTION = setSystemShell9x (what)
    END IF
END FUNCTION
