Attribute VB_Name = "modINI"
' Copyright 2004-2010 Erik Siers
' Provided under terms of the GNU GPL; see the file COPYING.TXT for details.

' ini module for EEShell2
' requires:
'   modAPI
' required by:
'   frmBG
'   frmOpts
'   modMain

Option Explicit


Public Sub writeINI()
    ' writes EEShell2.ini
    ' calls:
    '   writeINIValue   (modAPI->EEShell2.dll->Shell.bi)
    ' called by:
    '   frmBG.cmdExit_MouseUp
    '   frmBG.cmdFileMan_MouseUp
    '   frmBG.cmdLogoff_MouseUp
    '   frmBG.cmdRestart_MouseUp
    '   frmBG.cmdRun_MouseUp
    '   frmBG.cmdShutdown_MouseUp
    '   frmBG.Form_Unload
    '   frmBG.fraImgNav_MouseUp
    '   frmBG.mnuPopupButtonDefaults_Click
    '   frmOpts.cmdOK_Click

    '*****TO DO*****
    ' Rewrite this train wreck. readINI might need to be like this, but not writeINI.

    ' What I mean is that while it makes sense to read in the entire .ini file at once, it doesn't make sense to
    ' rewrite the entire thing every time a single setting gets changed. Why did I do this?

    ' The entire .ini should probably get rewritten when we exit the app, and probably when changing things via the
    ' Options dialog, but otherwise, it would make better sense to just write what's been changed. Doing so would also
    ' remove a layer of redirection. (This entire function is really just a wrapper for writeINIValue in the DLL,
    ' which is itself just a wrapper for the API call WritePrivateProfileString.)

    ' I should probably unexport readINIValue and writeINIValue and do everything outside of the DLL. There's no real
    ' performance gain by doing it in PB, but there *is* a definite (albeit tiny) performance hit by adding a DLL call
    ' on top of the API call. The only reason I do it via the DLL right now is because the DLL needs to write to
    ' system.ini under Win9x when changing the shell.

    ' Changing this will be a lot of immediate work in exchange for a long-term gain (I think).
    ' I'll do it, but not today.

    ' Also, I need to unroll the GOSUBs in both functions.

    Dim retval As Long, section As String, key As String, Value As String

    section = "EEShell2"
    key = "ConfirmExit"
        Value = IIf(confirmExit, "Y", "N")
        GoSub writer
    key = "FileManager"
        If Len(fileManPath) Then
            Value = fileManPath
        Else
            Value = "Explorer.exe"
        End If
        GoSub writer
    key = "Run"
        Value = IIf(runDialog, "Y", "N")
        GoSub writer
    key = "Shell"
        Value = IIf(isShell, "Y", "N")
        GoSub writer
    key = "StartMenuStartup"
        Value = IIf(startMenuStartup, "Y", "N")
        GoSub writer
    key = "SystrayIcon"
        Value = IIf(showSystray, "Y", "N")
        GoSub writer

    section = "Background"
    key = "Image"
        Value = bgImgName
        GoSub writer
    If randTime > 0 Then
        key = "Time"
        Value = CStr(randTime)
        GoSub writer
    End If

    With frmBG
        section = "ImageNavigator"
        With .fraImgNav
            key = "Show"
                Value = IIf(showImgNav, "Y", "N")
                GoSub writer
            key = "Left"
                Value = CStr(.Left)
                GoSub writer
            key = "Top"
                Value = CStr(.Top)
                GoSub writer
        End With

        section = "Buttons"
        key = "Exit"
            Value = IIf(showExit, "Y", "N")
            GoSub writer
        key = "Lock"
            Value = IIf(lockButtons, "Y", "N")
            GoSub writer
        key = "ExitLeft"
            Value = CStr(.cmdExit.Left)
            GoSub writer
        key = "ExitTop"
            Value = CStr(.cmdExit.Top)
            GoSub writer
        key = "FileManLeft"
            Value = CStr(.cmdFileMan.Left)
            GoSub writer
        key = "FileManTop"
            Value = CStr(.cmdFileMan.Top)
            GoSub writer
        key = "LogoffLeft"
            Value = CStr(.cmdLogoff.Left)
            GoSub writer
        key = "LogoffTop"
            Value = CStr(.cmdLogoff.Top)
            GoSub writer
        key = "RestartLeft"
            Value = CStr(.cmdRestart.Left)
            GoSub writer
        key = "RestartTop"
            Value = CStr(.cmdRestart.Top)
            GoSub writer
        key = "RunLeft"
            Value = CStr(.cmdRun.Left)
            GoSub writer
        key = "RunTop"
            Value = CStr(.cmdRun.Top)
            GoSub writer
        key = "ShutdownLeft"
            Value = CStr(.cmdShutdown.Left)
            GoSub writer
        key = "ShutdownTop"
            Value = CStr(.cmdShutdown.Top)
            GoSub writer
    End With

    Exit Sub

writer:
    ' I do it this way do I don't have to continually pass Values back-n-forth,
    ' just access them directly as needed and DAMN THE STACK SPACE!
    retval = writeINIvalue(App.Path & "\EEShell2.ini", section, key, Value)
    Return
End Sub


Public Sub readINI()
    ' reads all Values from EEShell2.ini
    ' calls:
    '   getINIValue (modAPI->EEShell2.dll->Shell.bi)
    ' called by:
    '   Main        (modMain)

    Dim retval As Long, section As String, key As String
    Dim v2 As String, Value As String, numeric As Boolean

    numeric = False

    section = "EEShell2"
    key = "StartMenuStartup"
        GoSub reader
        startMenuStartup = IIf("Y" = UCase$(Value), True, False)
    key = "ConfirmExit"
        GoSub reader
        confirmExit = IIf("Y" = UCase$(Value), True, False)
    With frmBG
        key = "FileManager"
            GoSub reader
            fileManPath = Value
        key = "Run"
            GoSub reader
            runDialog = IIf("N" = UCase$(Value), False, True)
        key = "Shell"
            GoSub reader
            isShell = IIf("Y" = UCase$(Value), True, False)
        key = "SystrayIcon"
            GoSub reader
            showSystray = IIf("N" = UCase$(Value), False, True)
    
    section = "Background"
    key = "Image"
        GoSub reader
        bgImgName = Value
    key = "Time"
        GoSub reader
        randTime = Val(Value)

    section = "ImageNavigator"
        With .fraImgNav
            key = "Show"
                GoSub reader
                showImgNav = IIf("Y" = UCase$(Value), True, False)
                .Visible = IIf("\" = Right$(bgImgName, 1), showImgNav, False)

            numeric = True

            key = "Left"
                GoSub reader
                .Left = IIf("-1" = Value, 120, Val(Value))
            key = "Top"
                GoSub reader
                .Top = IIf("-1" = Value, 1920, Val(Value))
        End With
        
        With .mnuPopupImgNav
            .Caption = IIf(showImgNav, "Hide &Image Navigator", "Show &Image Navigator")
            .Visible = ("\" = Right$(bgImgName, 1))
        End With

        section = "Buttons"
        numeric = False
        key = "Exit"
            GoSub reader
            If "Y" = UCase$(Value) Then
                showExit = True
            Else
                showExit = False
            End If
            .cmdExit.Visible = showExit
        key = "Lock"
            GoSub reader
            lockButtons = IIf("Y" = UCase$(Value), True, False)
        numeric = True
        key = "ExitLeft"
            GoSub reader
            .cmdExit.Left = IIf("-1" = Value, 120, Val(Value))
        key = "ExitTop"
            GoSub reader
            .cmdExit.Top = IIf("-1" = Value, 1560, Val(Value))
        key = "FileManLeft"
            GoSub reader
            .cmdFileMan.Left = IIf("-1" = Value, 120, Val(Value))
        key = "FileManTop"
            GoSub reader
            .cmdFileMan.Top = IIf("-1" = Value, 1560, Val(Value))
        key = "LogoffLeft"
            GoSub reader
            .cmdLogoff.Left = IIf("-1" = Value, 120, Val(Value))
        key = "LogoffTop"
            GoSub reader
            .cmdLogoff.Top = IIf("-1" = Value, 480, Val(Value))
        key = "RestartLeft"
            GoSub reader
            .cmdRestart.Left = IIf("-1" = Value, 120, Val(Value))
        key = "RestartTop"
            GoSub reader
            .cmdRestart.Top = IIf("-1" = Value, 840, Val(Value))
        key = "RunLeft"
            GoSub reader
            .cmdRun.Left = IIf("-1" = Value, 120, Val(Value))
        key = "RunTop"
            GoSub reader
            .cmdRun.Top = IIf("-1" = Value, 120, Val(Value))
        key = "ShutdownLeft"
            GoSub reader
            .cmdShutdown.Left = IIf("-1" = Value, 120, Val(Value))
        key = "ShutdownTop"
            GoSub reader
            .cmdShutdown.Top = IIf("-1" = Value, 1200, Val(Value))
        .resizeForm
    End With

    Exit Sub

reader:
    retval = getINIvalue(App.Path & "\EEShell2.ini", section, key, v2)
    If ("" = v2) Then
        If numeric Then
            v2 = "-1"
        Else
            retval = 0
        End If
    End If
    Value = v2 ' Left$(v2, retval)
    Return
End Sub
