Attribute VB_Name = "dllImageLister"
' Image selection routines for EEShell2.dll
' no external requirements

' Copyright 2004-2010 Erik Siers
' Provided under terms of the GNU GPL; see the file COPYING.TXT for details.

' This is only used in EEShell2-noDLL and EEShell2/16.

Option Explicit


Private fileLs() As String
Private currentImage As Long, totalImages As Long


Public Sub setInitialValues()
    ' sets shared variables to initial values
    ' noDLL only
    ' called by:
    '   modMain.Main

    currentImage = -1
    totalImages = -1
End Sub


Public Sub closeImages()
    ' erases fileLs prior to unloading DLL
    ' called by:
    '   frmBG.Form_Unload

    Erase fileLs()
End Sub


Sub recurseDirs(ByVal folder As String, chop As Long, ByRef UBfileLS As Long)
    ' follows a single branch all the way (stack space, don't fail me now)
    ' args:
    '   folder      folder we're working in
    '   chop        how much of the lead to chop off before jamming into fileLS$
    '   UBfileLS    ubound(fileLS$)
    ' calls:
    '   fixSlash
    '   recurseDirs ***RECURSIVE***
    '   (might need to make some API calls before I'm done...)
    ' called by:
    '   makeImgList

    ' I can also do this in VB with the FileSystemObject but it's too slow for my tastes...

    On Error GoTo Whoops

    Dim tmpStr1 As String, tmpLng1 As Long, fldrs As Long, extn As String, lFiles As Long
    Dim subDirs() As String, localFiles() As String

    fldrs = -1: lFiles = -1
    fixSlash folder

    tmpStr1 = Dir$(folder, FILE_ATTRIBUTE_DIRECTORY)
    Do Until "" = tmpStr1
        If ("." = tmpStr1) Or (".." = tmpStr1) Then GoTo looper
        tmpLng1 = GetAttr(folder & tmpStr1)
        If FILE_ATTRIBUTE_DIRECTORY = (tmpLng1 And FILE_ATTRIBUTE_DIRECTORY) Then
            fldrs = fldrs + 1
            ReDim Preserve subDirs(fldrs)
            subDirs(fldrs) = folder & tmpStr1
        Else
            'build localFiles, to be appended to fileLs later
            tmpLng1 = InStrRev(tmpStr1, ".")
            If 0 = tmpLng1 Then GoTo looper
            extn = LCase$(Mid$(tmpStr1, tmpLng1 + 1))
            Select Case extn
#If 1 = usePNG Then
                Case "bmp", "emf", "gif", "jpe", "jpeg", "jpg", "pcx", "png", "wmf"
#Else
                Case "bmp", "emf", "gif", "jpe", "jpeg", "jpg", "wmf"
#End If
                    lFiles = lFiles + 1
                    ReDim Preserve localFiles(lFiles)
                    localFiles(lFiles) = tmpStr1
            End Select
        End If
looper:
        tmpStr1 = Dir$
    Loop

    If lFiles > -1 Then
        ArraySort localFiles()
        ReDim Preserve fileLs(UBfileLS + lFiles + 1)
        tmpStr1 = Mid$(folder, chop)
        For tmpLng1 = 0 To lFiles
            fileLs(UBfileLS + tmpLng1 + 1) = tmpStr1 & localFiles(tmpLng1)
        Next
        UBfileLS = UBfileLS + lFiles + 1
    End If

    If fldrs > -1 Then
        ArraySort subDirs()
        For tmpLng1 = 0 To fldrs
            recurseDirs subDirs(tmpLng1), chop, UBfileLS
        Next
    End If

Whoops:
    Err.Clear
End Sub


Public Function makeImgList(Path As String) As Long
    ' starts the ball rolling on building the list
    ' args:
    '   path    base image path
    ' calls:
    '   recurseDirs
    ' called by:
    '   checkList

    On Error GoTo noimages
    Dim c As Long, lh As Long
    Dim cdir As String, lc As Long, c1 As String
    Dim fi As Long

    Erase fileLs()

    cdir = Path
    fixSlash cdir
    lc = Len(cdir) + 1

    c = -1
    lh = Len(cdir) + 1
    recurseDirs cdir, lh, c
    If c < 0 Then Exit Function

    c1 = CurDir$
    ChDir cdir
    fi = FreeFile
    totalImages = c + 1
noimages:
    ChDir c1
    makeImgList = Err.Number
    Err.Clear
End Function


Public Function checkList(Path As String) As Long
    ' makes sure the list exists; if not, rebuilds it
    ' args:
    '   path    base image path
    ' calls:
    '   makeImgList
    ' called by:
    '   nextPic
    '   prevPic
    '   randPic

    Static p As String
    Dim tmpLng1 As Long
    fixSlash Path
    Path = UCase$(Path)
    If (p <> Path) Or (totalImages < 0) Then
        p = Path
        tmpLng1 = makeImgList(p)
        checkList = tmpLng1
    End If
End Function


Public Function randPic(Path As String) As String
    ' selects a random entry in the image list
    ' args:
    '   path    base image path
    ' calls:
    '   checkList
    ' called by:
    '   frmBG.cmdImgRand_Click
    '   frmBG.resizeForm
    '   frmBG.tmrRandImg_Timer
    '   frmOpts.cmdBrowse1_Click
    '   frmOpts.Form_Load

    Dim l As Long
    l = checkList(Path)
    If l > 0 Then Exit Function
    Randomize Timer
    fixSlash Path
    currentImage = Int(Rnd * totalImages)
    randPic = Path & fileLs(currentImage)
End Function


Public Function nextPic(Path As String) As String
    ' selects the next entry in the image list, wrapping to 0 if at ubound
    ' args:
    '   path    base image path
    ' calls:
    '   checkList
    ' called by:
    '   frmBG.cmdImgNext_Click

    Dim l As Long
    l = checkList(Path)
    If l > 0 Then Exit Function
    currentImage = currentImage + 4
    If currentImage >= totalImages Then currentImage = 0
    nextPic = Path & fileLs(currentImage)
End Function


Public Function prevPic(Path As String) As String
    ' selects the previous entry in the image list, wrapping to ubound if at 0
    ' args:
    '   path    base image path
    ' calls:
    '   checkList
    ' called by:
    '   frmBG.cmdImgPrev_Click

    Dim l As Long
    l = checkList(Path)
    If l > 0 Then Exit Function
    currentImage = currentImage - 1
    If currentImage < 0 Then currentImage = totalImages - 1
    prevPic = Path & fileLs(currentImage)
End Function


' this isn't used anywhere, so commenting out
' if you need it, uncomment and recompile
'Function getCurrentPic() As String
'    getCurrentPic = fileLs(currentImage)
'End Function
