Attribute VB_Name = "dllShell"
' the shell module for EEShell2.dll
' requires:
'   dllMain.bas
'   dllWin32API.bas
' required by:
'   dllStartup.bas

' Copyright 2004-2010 Erik Siers
' Provided under terms of the GNU GPL; see the file COPYING.TXT for details.

' This is only used in EEShell2-noDLL and EEShell2/16.

Option Explicit

Private Const SYSBootKey = "SOFTWARE\Microsoft\Windows NT\CurrentVersion\IniFileMapping\system.ini\boot"
Private Const BootKeyShellValue = "Shell"


Function GetKeyValue(KeyRoot As Long, KeyName As String, SubKeyRef As String, _
        ByRef KeyVal As String) As Long
    ' retreives a registry value (wrapper for RegQueryValueEx)
    ' args:
    '   KeyRoot     the reg hive
    '   KeyName     the reg path
    '   SubKeyRef   the key name
    '   KeyVal      this is where the returned information gets stuffed
    ' calls:
    '   appLog          (dllMain.bas)
    '   FormatMessage   (dllWin32API.bas->kernel32)
    '   RegOpenKeyEx    (dllWin32API.bas->advapi32)
    '   RegQueryValueEx (dllWin32API.bas->advapi32)
    '   RegCloseKey     (dllWin32API.bas->advapi32)
    ' called by:
    '   getSystemShellNT
    '   setSystemShellNT

    Dim rc As Long
    Dim hKey As Long
    Dim hDepth As Long
    Dim KeyValType As Long
    Dim tmpVal As String * 256
    Dim KeyValSize As Long

    ' Open RegKey Under KeyRoot {HKEY_LOCAL_MACHINE...}
    rc = RegOpenKeyEx(KeyRoot, KeyName, 0, KEY_ALL_ACCESS, hKey)

    If (rc <> ERROR_SUCCESS) Then GoTo GetKeyError

    KeyValSize = 255

    ' Retrieve Registry Key Value...
    rc = RegQueryValueEx(hKey, SubKeyRef, 0, KeyValType, tmpVal, KeyValSize)

    If (rc <> ERROR_SUCCESS) Then GoTo GetKeyError

    ' Determine Key Value Type For Conversion...
    Select Case KeyValType
        Case REG_SZ
            KeyVal = tmpVal
        Case Else
            Error 151
    End Select

    rc = RegCloseKey(hKey)
    GetKeyValue = -1
    Exit Function

GetKeyError:
    hDepth = Err.Number
    Err.Clear
    Dim a1 As String, a2 As String
    a1 = Str$(KeyRoot)
    a2 = KeyName
    If hDepth Then
        tmpVal = Error$(hDepth)
        appLog "Error " & Str$(hDepth) & " while reading " & a1 & "\" & a2 & ": " & tmpVal & " (rc=" & Str$(rc) & ")"
    Else
        Dim a3 As Long
        a3 = 0
        FormatMessage FORMAT_MESSAGE_FROM_SYSTEM, a3, Err.LastDllError, 0, tmpVal, 255, a3
        appLog "Error while reading " & a1 & "\" & a2 & ": " & Trim$(tmpVal) & " (rc=" & Str$(rc) & ")"
    End If
    rc = RegCloseKey(hKey)
    KeyVal = "ERROR"
    GetKeyValue = 0
End Function


Function SetKeyValue(KeyRoot As Long, KeyName As String, SubKeyRef As String, ByRef KeyVal As String) _
        As Long
    ' sets a registry value (wrapper for RegSetValueEx)
    ' args:
    '   KeyRoot     the reg hive
    '   KeyName     the reg path
    '   SubKeyRef   the key name
    '   KeyVal      the new value
    ' calls:
    '   appLog          (dllMain.bas)
    '   FormatMessage   (dllWin32API.bas->kernel32)
    '   RegOpenKeyEx    (dllWin32API.bas->advapi32)
    '   RegSetValueEx   (dllWin32API.bas->advapi32)
    '   RegCloseKey     (dllWin32API.bas->advapi32)
    ' called by:
    '   setSystemShellNT

    Dim rc As Long
    Dim hKey As Long
    Dim hDepth As Long
    Dim KeyValType As Long
    Dim tmpVal As String * 256
    Dim KeyValSize As Long

    ' Open RegKey Under KeyRoot {HKEY_LOCAL_MACHINE...}
    rc = RegOpenKeyEx(KeyRoot, KeyName, 0, KEY_ALL_ACCESS, hKey)

    If (rc <> ERROR_SUCCESS) Then GoTo GetKeyError

    KeyValSize = 255

    ' Set Registry Key Value...
    KeyValType = REG_SZ
    rc = RegSetValueEx(hKey, SubKeyRef, 0, KeyValType, KeyVal, KeyValSize)

    If (rc <> ERROR_SUCCESS) Then GoTo GetKeyError

    rc = RegCloseKey(hKey)
    SetKeyValue = -1
    Exit Function

GetKeyError:
    hDepth = Err.Number
    Err.Clear
    Dim a1 As String, a2 As String
    a1 = Str$(KeyRoot)
    a2 = KeyName
    If hDepth Then
        tmpVal = Error$(hDepth)
        appLog "Error " & Str$(hDepth) & " while reading " & a1 & "\" & a2 & ": " & tmpVal & " (rc=" & Str$(rc) & ")"
    Else
        Dim a3 As Long
        a3 = 0
        FormatMessage FORMAT_MESSAGE_FROM_SYSTEM, a3, Err.LastDllError, 0, tmpVal, 255, a3
        appLog "Error while reading " & a1 & "\" & a2 & ": " & Trim$(tmpVal) & " (rc=" & Str$(rc) & ")"
    End If
    rc = RegCloseKey(hKey)
    KeyVal = "ERROR"
    SetKeyValue = 0
End Function


Function getINIvalue(file As String, section As String, key As String, value As String) As Long
    ' reads a value from a .ini file (wrapper for GetPrivateProfileString)
    ' args:
    '   file    the .ini file to look in
    '   section the [section] to look under
    '   key     the key= to look at
    '   value   this is where the returned information gets stuffed
    ' calls:
    '   GetPrivateProfileString (dllWin32API.bas->kernel32)
    ' called by:
    '   getSystemShell9x
    '   modINI.readINI

    Dim res As String * 256, resval As Long

    resval = GetPrivateProfileString(section, key, "", res, 256, file)
    value = Left$(res, resval)
    getINIvalue = resval
End Function


Function writeINIvalue(file As String, section As String, key As String, value As String) _
        As Long
    ' writes a value to a .ini file (wrapper for WritePrivateProfileString)
    ' args:
    '   file    the .ini file to look in
    '   section the [section] to look under
    '   key     the key= to look at
    '   value   what to assign to key
    ' calls:
    '   WritePrivateProfileString (dllWin32API.bas->kernel32)
    ' called by:
    '   setSystemShell9x
    '   modINI.writeINI

    Dim res As String * 256, resval As Long

    resval = WritePrivateProfileString(section, key, value, file)
    writeINIvalue = resval   ' 0 = failure, anything else is success
End Function


#If 0 Then
' this isn't used anywhere -- my original intention was to save the old shell name somewhere
Function getSystemShell9x() As String
    ' reads the current shell from %windir%\system.ini
    ' calls:
    '   getINIvalue
    ' called by:
    '   getSystemShell

    Dim res As String, tmpLng As Long
    Dim tmp1 As String * 5, tmp2 As String * 6, tmp3 As String * 11

    tmp1 = "boot"
    tmp2 = BootKeyShellValue
    tmp3 = "System.ini"

    tmpLng = getINIvalue(tmp3, tmp1, tmp2, res)
    getSystemShell9x = res
End Function
#End If


Function setSystemShell9x(what As String) As Long
    ' sets the shell in %windir%\system.ini
    ' args:
    '   what    what to set as the new shell
    '           Win32: should be "Explorer.exe" or the full path to EEShell2
    '           Win16: should be "Progman.exe" or the full path to EEShell2/16
    ' calls:
    '   writeINIvalue
    ' called by:
    '   setSystemShell

    Dim retval As Long
    Dim tmp1 As String * 5, tmp2 As String * 6, tmp3 As String * 11

    tmp1 = "boot"
    tmp2 = BootKeyShellValue
    tmp3 = "System.ini"

    retval = writeINIvalue(tmp3, tmp1, tmp2, what)
    setSystemShell9x = retval
End Function


#If 0 Then
' this isn't used anywhere -- my original intention was to save the old shell name somewhere
Function getSystemShellNT() As String
    ' reads the current shell from the registry
    ' calls:
    '   GetKeyValue
    ' called by:
    '   getSystemShell

    Dim c As Long
    Dim currentShell As String * MAX_PATH, usrShell As String * MAX_PATH

    c = GetKeyValue(HKEY_LOCAL_MACHINE, SYSBootKey, BootKeyShellValue, currentShell)
    If c Then
        Select Case UCase$(Left$(currentShell, 3))
            Case "USR"
                c = GetKeyValue(HKEY_CURRENT_USER, currentShell, BootKeyShellValue, usrShell)
                If c Then currentShell = usrShell
            Case "SYS"
                c = GetKeyValue(HKEY_LOCAL_MACHINE, currentShell, BootKeyShellValue, usrShell)
                If c Then currentShell = usrShell
            Case Else
                currentShell = "ERROR"
        End Select
        getSystemShellNT = currentShell
    End If
End Function
#End If


Function setSystemShellNT(what As String) As Long
    ' changes the shell in the registry
    ' args:
    '   what    the new shell
    '           should be either "Explorer.exe" or the full path to EEShell2
    ' calls:
    '   GetKeyValue
    '   SetKeyValue
    ' called by:
    '   setSystemShell

    Dim c As Long
    Dim currentShell As String * MAX_PATH, usrShell As String * MAX_PATH

    c = GetKeyValue(HKEY_LOCAL_MACHINE, SYSBootKey, BootKeyShellValue, currentShell)
    If c Then
        Select Case UCase$(Left$(currentShell, 3))
            Case "USR"
                c = SetKeyValue(HKEY_CURRENT_USER, "Software\Microsoft\Windows NT\CurrentVersion\Winlogon", BootKeyShellValue, what)
                appLog "Set EEShell2 as shell."
                setSystemShellNT = c
            Case "SYS"
                If ("EXPLORER.EXE" = UCase$(what)) Or ("EXPLORER" = UCase$(what)) Then
                    ' I expect this to fail if the current user isn't admin
                    c = SetKeyValue(HKEY_LOCAL_MACHINE, "Software\Microsoft\Windows NT\CurrentVersion\Winlogon", BootKeyShellValue, what)
                    setSystemShellNT = c
                Else
                    MsgBox "EEShell2 will not set itself as the shell for all users under NT systems." & vbCrLf & _
                           "Import the registry file ""1-admin-install.reg"" (in the EEShell2 directory)" & vbCrLf & _
                           "and then try again.", vbCritical Or vbSystemModal
                    appLog "Failed to set EEShell2 as shell."
                    setSystemShellNT = 0    'failure
                End If
            Case Else
                MsgBox "An unexpected registry value was encountered while trying to set EEShell2 as the shell." & vbCrLf & _
                       "EEShell2 will not be able to automatically set itself as your shell on this system." & vbCrLf & _
                       "Read the file ""Shell.txt"" in the EEShell2 directory for manual instructions.", _
                       vbCritical Or vbSystemModal
                setSystemShellNT = 0    'failure
        End Select
    End If
End Function


#If 0 Then
' this isn't used anywhere -- my original intention was to save the old shell name somewhere
Function getSystemShell() As String
    ' wrapper for getSystemShell9x and getSystemShellNT
    ' calls:
    '   getSystemShell9x
    '   getSystemShellNT
    '   IsWinNT

    If IsWinNT Then
        getSystemShell = getSystemShellNT
    Else
        getSystemShell = getSystemShell9x
    End If
End Function
#End If


Function setSystemShell(what As String) As Long
    ' changes the shell is a system-specific manner
    ' (wrapper for setSystemShell9x and setSystemShellNT)
    ' args:
    '   what    the new shell
    ' calls:
    '   IsWinNT             (dllMain.bas)
    '   setSystemShell9x
    '   setSystemShellNT
    ' called by:
    '   frmOpts.cmdOK_Click

    If IsWinNT Then
        setSystemShell = setSystemShellNT(what)
    Else
        setSystemShell = setSystemShell9x(what)
    End If
End Function
